function [W, H, E] = nmf_is(V, W0, H0, iter_num)

%
% [W, H, E] = nmf_is(V, W0, H0, iter_num);
%
% NMF decomposition using multiplicative update (MU) rules and
% Itakura-Saito (IS) divergence
%
%
% input 
% -----
%
% V                 : nonnegative data matrix to factorize (V = W*H) [F x N]
% W0                : initial matrix of bases [F x K]
% H0                : initial matrix of contributions [K x N]
% iter_num          : (opt) number of EM iterations (def = 100)
%
% output
% ------
%
% W                 : estimated matrix of bases [F x K]
% H                 : estimated matrix of contributions [K x N]
% E                 : vector of approximation errors (per iteration)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if nargin < 4 || isempty(iter_num)
    iter_num = 100;
end;

[F,N] = size(V);

W = W0;
H = H0;

S = sum(V(:));
matOfOnes = ones(F,N);
WH = W*H + eps;

E = zeros(1, iter_num);

for iter = 1:iter_num
    % update of W
    W = W .* (( V./((WH).^2) )*H' + eps) ./ ((1./(WH))*H' + eps);
    
    % normalization
    w = sum(W,1); 
    d = diag(1./w); 
    W = W*d; % Normalizatiosn of columns of W
    d = diag(w); 
    H = d*H; % Energy transfer to H
    
    WH = W*H + eps;
    
    % update of H
    H = H .* (W'*(V./((WH).^2)) + eps) ./ (W'*(1./(WH)) + eps);    
    WH = W*H + eps;

    % Error with IS divergence:
    E(iter) = sum(sum(V./(WH + eps) - log(V./(WH + eps) + eps) - matOfOnes))/S; % normalized error
    
%     fprintf('MU-IS-NMF: iteration %d of %d, approximation error = %f\n', iter, iter_num, E(iter));
end
