%==========================================================================
% Main script
%
% S. Leglaive, R. Badeau, G. Richard, "Multichannel Audio Source Separation: Variational Inference of
% Time-Frequency Sources from Time-Domain Observations", IEEE International Conference on
% Acoustics Speech and Signal Processing (ICASSP), 2017.

% Simon Leglaive, Télécom ParisTech, 2016
%==========================================================================

addpath('./aux_tools')
addpath('./mtimesx')

%% Global parameters

dataDir = './sounds'; % directory of true source images and mix
resultsDir = dataDir; % directory to write estimated source images
resultsPrefix = 'sim_VEM_';

J = 5; % number of sources
I = 2; % number of microphones
wlen = 2048;
hop = wlen/2;
niter = 50; % number of iterations for the GEM algorithm

wavFile = fullfile(dataDir,'mix.wav');
[x, fs] = audioread(wavFile);

T_orig = size(x,1);
T60 = 256; % reverberation time in ms
La = T60*1e-3*fs;
Ls = T_orig-La+1;

%% Oracle initialization

K = 10; % NMF order
[ a_init, W_init, H_init, sigmaNoise_init, ~, seedW, seedH ] = ...
    oracleInit( dataDir, Ls, wlen, K, 'mix_filters.mat', 1, 1);

%% Zero padding for handling edges

Nzp_end = ceil(Ls/hop)*hop-Ls;
Nzp_end = Nzp_end + wlen/2;
Nzp_beg = wlen/2;
Ls = Ls + Nzp_beg + Nzp_end;

x = [zeros(Nzp_beg, I) ; x ; zeros(Nzp_end, I)];
T = size(x,1);

F = wlen/2;
N = round(Ls/hop) - 1; % number of frames
win = sin((.5:wlen-.5)/wlen*pi).';

%% Run VEM algorithm

fprintf('Instantiation of VEMalgo\n');
ss_algoVEM = VEMalgo('x', x, 'a', a_init, 'sigmaNoise', sigmaNoise_init, 'W', {W_init}, 'H', {H_init});

varFreeEnergy = zeros(niter,1);

for iter = 1:niter
    fprintf('\nVEM iteration %d of %d\n\n', iter, niter);

    ss_algoVEM.EStepPCG;
    ss_algoVEM.MStep;
    ss_algoVEM.ComputeVarFreeEnergy(0);
    varFreeEnergy(iter) = ss_algoVEM.varFreeEnergy;

    fprintf('variational free energy: %.2f\n', varFreeEnergy(iter)/(I*T));


end
ss_algoVEM.EStepPCG;


%% Compute estimated source images

Ls = T_orig-La+1;
se = imdct_multi(ss_algoVEM.m_post, Ls);
sime = zeros(J, T_orig, I);
for j=1:J
    for i=1:I
    sime(j,:,i) = conv(se(:,j), squeeze(ss_algoVEM.a(i,j,:)));
    end
end

x_orig = x(Nzp_beg+1:Nzp_beg+T_orig,:);

for k=1:J,
    audiowrite(fullfile(resultsDir, [resultsPrefix num2str(k) '.wav']), reshape(sime(k,:,:),T_orig,2),fs);
end

%% Computation of the scores

fprintf('Computation of the scores\n');
sim_true = zeros(J,T_orig,2);
for j=1:J
    sim_true(j,:,:)=reshape(audioread(fullfile(dataDir, ['sim' int2str(j) '.wav'])),1,T_orig,2);
end
[SDR_VEM,ISR_VEM,SIR_VEM,SAR_VEM,perm_VEM] = bss_eval_images(sime,sim_true);

resVEM = [SDR_VEM,ISR_VEM,SIR_VEM,SAR_VEM,perm_VEM];
